# Summary

## Overview
* [Understanding MassTransit](understand/README.md)
    * [Key terminology](understand/key-ideas.md)
    * [Under the hood](understand/under-the-hood.md)
        * [Topology](understand/default-topology.md)
    * [What we add to the transport](understand/additions-to-transport.md)
    * [Publishing messages](understand/publishing.md)
    * [Sending messages](understand/sending.md)
    * [Message contract design](understand/message-contracts.md)
    * [Performance counters](understand/perfcounters.md)
* [Quick Start!](quickstart.md)
* [I need help!](introduction/getting-help.md)
* [Learning MassTransit](learn/README.md)
    * [Samples](learn/samples/README.md)
        * [Courier](learn/samples/courier.md)
        * [Request/Response](learn/samples/request-response.md)
        * [Saga](learn/samples/saga.md)
    * [Videos](learn/videos.md)
    * [Courses](learn/courses.md)
    * [Loving the community](learn/loving-the-community.md)
* [Contributing](contributing/README.md)

## Usage
* [Installation](introduction/README.md)
    * [Prerequisites](introduction/prerequisites.md)
    * [How to install](introduction/how-to-install.md)
    * [Getting help](introduction/getting-help.md)
    * [How to report bugs](introduction/how-to-report-bugs.md)

* [Using MassTransit](usage/README.md)
    * [Configure MassTransit](usage/configuration.md)
    * [Choose a transport](usage/transports.md)
    * [Create message contracts](usage/message-contracts.md)
    * [Create message consumers](usage/message-consumers.md)
    * [Produce messages](usage/producing-messages.md)
    * [Correlate messages](usage/correlation.md)
    * [Use IoC container](usage/containers/README.md)
        * [Autofac](usage/containers/autofac.md)
        * [Ninject](usage/containers/ninject.md)
        * [StructureMap](usage/containers/structuremap.md)
        * [Unity](usage/containers/unity.md)
        * [Castle Windsor](usage/containers/castlewindsor.md)
    * [Request/response](usage/request-response.md)
    * [Handle exceptions](usage/exceptions.md)
    * [Configure retries](usage/retries.md)
    * [Schedule messages](usage/scheduling/README.md)
        * [Scheduling API](usage/scheduling/scheduling-api.md)
        * [In-memory scheduling](usage/scheduling/in-memory.md)
        * [Azure Service Bus](usage/scheduling/azure-sb-scheduler.md)
        * [RabbitMQ Delayed Exchange](usage/scheduling/rabbitmq-delayed.md)
        * [Redelivering messages](usage/scheduling/redeliver.md)
    * [Observe messages](usage/observers.md)
    * [Observe lifecycle](usage/lifecycle-observers.md)
    * [Connect endpoints](usage/connect-endpoint.md)
    * [Logging](usage/logging.md)
    * [Testing](usage/testing/README.md)
        * [Bus harness](usage/testing/bus-harness.md)
        * [Testing consumers](usage/testing/testing-consumers.md)
        * [Testing sagas](usage/testing/testing-sagas.md)

## Advanced 
* [Sagas](advanced/sagas/README.md)
    * [Automatonymous](advanced/sagas/automatonymous.md)
    * [Persistence](advanced/sagas/persistence.md)
* [Courier](advanced/courier/README.md)
    * [Activities](advanced/courier/activities.md)
    * [Builder](advanced/courier/builder.md)
    * [Execution](advanced/courier/execute.md)
    * [Events](advanced/courier/events.md)
    * [Subscriptions](advanced/courier/subscriptions.md)
* [Middleware](advanced/middleware/README.md)
    * [Circuit breaker](advanced/middleware/circuit-breaker.md)
    * [Rate limiter](advanced/middleware/rate-limiter.md)
    * [Latest](advanced/middleware/latest.md)
    * [Custom](advanced/middleware/custom.md)
* [Audit](advanced/audit/README.md)
* [Turnout](advanced/turnout/README.md)
* [Versioning](advanced/versioning.md)
* [Topology](advanced/topology/README.md)
    * [Message](advanced/topology/message.md)
    * [Publish](advanced/topology/publish.md)
    * [Send](advanced/topology/send.md)
    * [Consume](advanced/topology/consume.md)
    * [Conventions](advanced/topology/conventions.md)
    * [RabbitMQ](advanced/topology/rabbitmq/README.md)
        * [Exchange Binding](advanced/topology/rabbitmq/binding.md)
        * [RoutingKey](advanced/topology/rabbitmq/routingkey.md)
    * [Azure Service Bus](advanced/topology/servicebus/README.md)
        * [SessionId](advanced/topology/servicebus/sessionid.md)
        * [PartitionKey](advanced/topology/servicebus/partitionkey.md)
        * [Topic Subscription](advanced/topology/servicebus/topicsub.md)
* [Transactions](advanced/transactions.md)
* [Interoperability](advanced/interoperability.md)

## Troubleshooting
* [Troubleshooting](troubleshooting/README.md)
    * [Common gotchas](troubleshooting/common-gotchas.md)
    * [Diagnostics](troubleshooting/show-config.md)

## Architecture
* [Articles](architecture/README.md)
    * [Green Cache](architecture/green-cache.md)
