﻿// Copyright 2007-2017 Chris Patterson, Dru Sellers, Travis Smith, et. al.
//  
// Licensed under the Apache License, Version 2.0 (the "License"); you may not use
// this file except in compliance with the License. You may obtain a copy of the 
// License at 
// 
//     http://www.apache.org/licenses/LICENSE-2.0 
// 
// Unless required by applicable law or agreed to in writing, software distributed
// under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
// CONDITIONS OF ANY KIND, either express or implied. See the License for the 
// specific language governing permissions and limitations under the License.
namespace MassTransit.Configuration
{
    using Pipeline;
    using Pipeline.Pipes;
    using PublishPipeSpecifications;
    using Topology;
    using Topology.Observers;


    public class PublishPipeConfiguration :
        IPublishPipeConfiguration
    {
        readonly PublishPipeSpecification _specification;

        public PublishPipeConfiguration(IPublishTopology publishTopology)
        {
            _specification = new PublishPipeSpecification();
            _specification.Connect(new TopologyPublishPipeSpecificationObserver(publishTopology));
        }

        public PublishPipeConfiguration(IPublishPipeSpecification parentSpecification)
        {
            _specification = new PublishPipeSpecification();
            _specification.Connect(new ParentPublishPipeSpecificationObserver(parentSpecification));
        }

        public IPublishPipeSpecification Specification => _specification;
        public IPublishPipeConfigurator Configurator => _specification;

        public IPublishPipe CreatePipe()
        {
            return new PublishPipe(_specification);
        }
    }
}