// Copyright 2007-2017 Chris Patterson, Dru Sellers, Travis Smith, et. al.
//  
// Licensed under the Apache License, Version 2.0 (the "License"); you may not use
// this file except in compliance with the License. You may obtain a copy of the 
// License at 
// 
//     http://www.apache.org/licenses/LICENSE-2.0 
// 
// Unless required by applicable law or agreed to in writing, software distributed
// under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
// CONDITIONS OF ANY KIND, either express or implied. See the License for the 
// specific language governing permissions and limitations under the License.
namespace MassTransit
{
    using System;
    using GreenPipes;
    using Saga;


    public interface ISagaMessageConfigurator<TMessage> :
        IPipeConfigurator<ConsumeContext<TMessage>>
        where TMessage : class
    {
    }


    public interface ISagaMessageConfigurator<TSaga, TMessage> :
        IPipeConfigurator<SagaConsumeContext<TSaga, TMessage>>
        where TMessage : class
        where TSaga : class, ISaga
    {
        /// <summary>
        /// Add middleware to the saga pipeline, for the specified message type, which is
        /// invoked after the saga repository.
        /// </summary>
        /// <param name="configure">The callback to configure the message pipeline</param>
        void Message(Action<ISagaMessageConfigurator<TMessage>> configure);
    }
}